<?xml version="1.0" encoding="iso-8859-1" standalone="no" ?>
<!-- DOCTYPE xsl:stylesheet SYSTEM ""-->

<xsl:stylesheet version="1.0"
  xml:space="default"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:bib="http://exmo.inrialpes.fr/papers"
  xmlns:date="http://www.jclark.com/xt/java/java.util.Date"
>

<xsl:output method="text" media-type="text/plain" encoding="iso-8859-1"/>

<xsl:strip-space elements="*"/>

<!-- toplevel -->

<xsl:template match="/">
% File generated by bibML2BibTeX on <xsl:if test="function-available('date:to-string') and function-available('date:new')"><xsl:value-of select="date:to-string(date:new())"/></xsl:if>
		<xsl:apply-templates/>
</xsl:template>

<xsl:template match="bibliography">
% <xsl:value-of select="@name"/> bibliography (<xsl:value-of select="@date"/>)
	<xsl:apply-templates select="*"/>
</xsl:template>

<xsl:template match="reference">
@<xsl:value-of select="@type"/>{<xsl:value-of select="@index"/>,<xsl:apply-templates/>
	<xsl:if test="@language">
	LANGUAGE = "<xsl:value-of select='@language'/>",</xsl:if>
	CREATEDBY = "BibML2BibTeX"}
</xsl:template>

<xsl:template match="authors"> <!-- // -->
	AUTHOR = "<xsl:apply-templates select='p[1]'/>
	<xsl:for-each select='p[position()>1]'> and <xsl:apply-templates select='.'/>
	</xsl:for-each>",</xsl:template>

<xsl:template match="editors">
	EDITOR = "<xsl:apply-templates select='p[1]'/>
	<xsl:for-each select='p[position()>1]'> and <xsl:apply-templates select='.'/>
	</xsl:for-each>",</xsl:template>

<xsl:template match="title">
	TITLE = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="publisher">
	PUBLISHER = "<xsl:value-of select='@name'/>",<xsl:if test="location">
	ADDRESS = "<xsl:apply-templates select='location'/>",</xsl:if></xsl:template>

<xsl:template match="type">
	TYPE = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="conference"> <!-- // -->
	<xsl:if test='@issue'>
	ISSUE = <xsl:value-of select='@issue'/>,</xsl:if>
	BOOKTITLE = "<xsl:value-of select='@type'/><xsl:text> </xsl:text><xsl:value-of select='@name'/>, <xsl:apply-templates select='location'/>",<xsl:if test="@sponsor">
	ORGANISATION = "<xsl:value-of select='@sponsor'/>",</xsl:if></xsl:template>

<xsl:template match="book">
	BOOKTITLE = "<xsl:value-of select='reference/title'/>",<xsl:if test="reference/authors">
	EDITOR = "<xsl:apply-templates select='reference/authors/p[1]'/><xsl:for-each select='reference/authors/p[position()>1]'> and <xsl:apply-templates select='.'/>
	</xsl:for-each>",</xsl:if><xsl:if test="reference/editors">
	EDITOR = "<xsl:apply-templates select='reference/editors/p[1]'/><xsl:for-each select='reference/editors/p[position()>1]'> and <xsl:apply-templates select='.'/>
	</xsl:for-each>",</xsl:if>
	<xsl:apply-templates select='reference/publisher'/></xsl:template>

<xsl:template match="organization|institution">
	INSTITUTION = "<xsl:value-of select='@name'/>",
	<xsl:if test="location">
	ADDRESS = "<xsl:apply-templates select="location"/>",</xsl:if></xsl:template>

<xsl:template match="school">
	SCHOOL = "<xsl:value-of select='@name'/>",<xsl:if test="location">
	ADDRESS = "<xsl:apply-templates select="location"/>",</xsl:if></xsl:template>

<xsl:template match="chapter">
	CHAPTER = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="howpublished">
	HOWPUBLISHED = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="journal">
	JOURNAL = "<xsl:value-of select='@name'/>",<xsl:if test="publisher"><xsl:apply-templates select="publisher"/>
	</xsl:if></xsl:template>

<xsl:template match="edition">
	EDITION = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="volume">
	VOLUME = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="number">
	NUMBER = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="pages">
	PAGES = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="year">
	YEAR = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="month">
	MONTH = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="address">
	ADDRESS = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="links"><xsl:apply-templates select="url[1]"/></xsl:template>

<xsl:template match="url">
	URL = "<xsl:value-of select='@href'/>",</xsl:template>

<xsl:template match="series">
	SERIES = "<xsl:value-of select='@name'/>",</xsl:template>

<xsl:template match="note">
	NOTE = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="annote">
	ANNOTE = "<xsl:value-of select='normalize-space()'/>",</xsl:template>

<xsl:template match="p">{<xsl:value-of select="@last"/>}, {<xsl:value-of select="@first"/>}</xsl:template>

<xsl:template match="location">
	<xsl:if test="@place"><xsl:value-of select="@place"/><xsl:text>, </xsl:text></xsl:if>
	<xsl:if test="@city"><xsl:value-of select="@city"/></xsl:if>
	<xsl:if test="@country"><xsl:text> (</xsl:text>
		<xsl:if test="@state"><xsl:value-of select="@state"/><xsl:text> </xsl:text></xsl:if>
		<xsl:value-of select="@country"/>)</xsl:if></xsl:template>

<!-- this could be put back now -->
<xsl:template match="areas|softwares|contracts|keywords|abstract|date"/>

<xsl:template match="text()"/>

<xsl:template match="li"><xsl:value-of select="normalize-space()"/></xsl:template>

</xsl:stylesheet>
