/**
 * $Id: transmorph.java,v 1.15 2001/12/20 12:08:32 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * Parse and print and XML speification for Transmorpher.
 * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
 * @author Jrme Euzenat (Jerome.Euzenat@inrialpes.fr)
 * @since SAX 2.0 / jdk 1.2 / XALAN 2.0
 */

package fr.fluxmedia.transmorpher.Application ;

import fr.fluxmedia.transmorpher.Parser.ProcessParser ;
import fr.fluxmedia.transmorpher.Parser.FMParser ;
import fr.fluxmedia.transmorpher.Graph.Transmorpher ;
import fr.fluxmedia.transmorpher.Utils.SystemResources ;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.Version;

import java.util.Enumeration;

public final class transmorph {

    static int debug_mode = 0;


   /**
     * Process a call to a function.
     * This is the main function of the application.
     *
     * @ param the name of the XML file which describe the
     * execution process
     */

  public static final void main(String argc[]) {
    System.setErr(System.out);
    boolean time = true ;     
    double begin = System.currentTimeMillis();
    double end = 0;
    SystemResources vSR= new SystemResources();
    
   
    if (debug_mode > 0) System.err.println(">> Getting parameters");
    CommandLineArgument arg = new CommandLineArgument();
    arg.argsAnalyse(argc);
    debug_mode = arg.getDebugMode();
    if ( debug_mode > 0 && arg.IsOk() ) arg.printVersion();

    if (debug_mode > 1){
      for( Enumeration e = arg.getParameters().getNames() ; e.hasMoreElements(); ){
        String key = (String)e.nextElement();
        String val = (String)arg.getParameters().getParameter(key);
        System.err.println(" +++ "+key+" = "+val);
      } //end for
    }
 
    if ( arg.IsOk() ){ // no continuation: error detected in command line.
	if ( arg.getFilename() != null ){
	    try {
		FMParser iProcessParser = new ProcessParser(debug_mode);
      
		if (debug_mode > 0) System.err.println(">> Begin graph construction");
		Transmorpher graph = iProcessParser.Newparse(arg.getFilename());
		if (debug_mode > 0) {
		    end = System.currentTimeMillis() ;
		    System.err.println(">> End of graph construction ("+(end-begin)+")");}
		if ( arg.getExecMode()) {
		    if (debug_mode > 0) {
			System.err.println(">> Begining of Execution generation");
			begin = System.currentTimeMillis();}
		    graph.generateExec(arg.getRelocation());
		    if (debug_mode > 0) {
			end = System.currentTimeMillis() ;
			System.err.println(">> End of Execution generation ("+(end-begin)+")");
			System.err.println(">> Begining of Execution");
			begin = System.currentTimeMillis();}
		    graph.exec(arg.getParameters());
		    if (debug_mode > 0) {
			end = System.currentTimeMillis() ;
			System.err.println(">> End of execution ("+(end-begin)+")");}
		} else if ( arg.getRoundMode() ) {
		    graph.generateXML();
		} else if ( arg.getTestMode() ) {
		    //fr.fluxmedia.transmorpher.Files.CharEngine CE= new fr.fluxmedia.transmorpher.Files.CharEngine("../samples/biblio/input/bibexmo.xml");
		    //CE.ReadXML();
		} else { // compile mode
		    if (debug_mode > 0) {
			System.err.println(">> Begin of code generation");
			begin = System.currentTimeMillis();}
		    graph.generateJavaCode(arg.getRelocation());
		    if (debug_mode > 0) {
			end = System.currentTimeMillis() ;
			System.err.println(">> End of code generation ("+(end-begin)+")");}
		}
	    } catch (Exception e) {
		System.err.println("transmorph : Erreur ["+ e +"]");
		e.printStackTrace();
	    } //end try
	} else {
	    System.err.println("transmorph : missing transformation filename");
	    arg.printHelp();
	} // end If
    } // end If
  } //end main
    
} //end class
