/**
 * $Id: ApplyProcess.java,v 1.11 2001/12/20 12:13:34 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.ListIterator;

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct ;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;
import fr.fluxmedia.transmorpher.TProcessFactory.TApplyProcess;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcess;

public class ApplyProcess
  extends ApplyImpl {

  public ApplyProcess (String n, String t, Process p, String pr, int in, int out){
    super(n, t, p, pr, in, out);
  }

  /** ask a nullify call to the called process */
  public void retroNull( PortList out, PortList in) {
      // JE: check is it is a Process
      Process called = (Process)(getProcess().getTransmorpher().getTransformation(getRef()));
      nullified = called.retroNull(out, in, true);
      visited = true;
  }
  
  public void generateXML (){
    System.out.print("      <apply-process id=\""+id+"\" ref=\""+getRef()+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    if ( parameters.isEmpty() ){
      System.out.println("\"/>");
    } else {
      System.out.println("\">");
      parameters.generateXMLCalls();
      System.out.println("      </apply-process>");
    }
  } //end generate XML

    public final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory){

    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    // get the name corresponding to the generated stylesheet and pas it in parameter
    String ref = getAttributes().getStringParameter("ref");
    Transmorpher t = getProcess().getTransmorpher() ;
    LinearIndexedStruct vCalls = ((Process)(t.getTransformation(ref))).getCalls();
    //System.out.println("[ExecutionStructure]ApplyProcess : getTransformation"
    //                    +((fr.fluxmedia.transmorpher.Graph.Process)iTransmorpher.getTransformation(ref)));
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newApplyProcess(vPortIn,vPortOut,
							getAttributes().getStringParameter("type"),
                                                        getParameters(),
                                                        getAttributes());
    PortList LocIn = t.getTransformation(ref).inPorts();
    PortList LocOut= t.getTransformation(ref).outPorts();
    ((TApplyProcess)vCurrent).setLocalName(LocIn.getNames(),LocOut.getNames());
            
    for( ListIterator l = vCalls.listIterator(); l.hasNext() ; ){
     	((Call)l.next()).createProcess(vCurrent,iProcessFactory);
    }
    currentProcess.addProcess(vCurrent);
  } //end proc

  public void setUp(){
    if( getProcess().getTransmorpher().getTransformation(getRef()) == null ){
      System.err.println("warning : unknown process : "+getRef());
    }
  } //end setUp
   
    /** This is the main problematic aspect: 
	There is a possible recursion in Process calls: hey should thus be stacked for working correctly
    **/
   public void generateJavaCode(ReadWrite file){
    if(file == null) {
      System.out.println(getId()+" [generateCode]: the file is null!!");
    } else {
      String r = getAttributes().getStringParameter("ref");
      Transmorpher tr = getProcess().getTransmorpher() ;
      fr.fluxmedia.transmorpher.Graph.Process processDef = ((fr.fluxmedia.transmorpher.Graph.Process)tr.getTransformation(r));
      file.Writeln();
      file.Writeln(4,"//------------------ Generation of ApplyProcess "+getId()+" -------------------");
      generatePorts(file);
      generateParameters(file);
      file.Writeln(4,"tmCurrentProcess = tmProcessFactory.newApplyProcess(portIn,portOut,\""+getType()+"\",tmParameters,tmAttributes);");
      file.Writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
      file.Writeln();
      file.Writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
      file.Writeln();
      // JE: Here we stack the tmCurrentCompositeProcess
      file.Writeln(4,"tmProcessStack.push((Object)tmCurrentCompositeProcess);");
      file.Writeln(4,"tmCurrentCompositeProcess = (TProcessComposite)tmCurrentProcess;");
      file.Writeln();
      // JE: Change of port names ?
      file.Writeln(4,"LocIn = new String["+ processDef.inPorts().getNames().length+"];");
      for(int i=0;i<processDef.inPorts().length();i++){
	  file.Writeln(4,"LocIn["+i+"]=\""+processDef.inPorts().getNames()[i]+"\";");
      }
      file.Writeln(4,"LocOut = new String["+processDef.outPorts().getNames().length+"];");
      for(int i=0;i<processDef.outPorts().length();i++){
	  file.Writeln(4,"LocOut["+i+"]=\""+processDef.outPorts().getNames()[i]+"\";");
      }
      file.Writeln(4,"((TApplyProcess)tmCurrentProcess).setLocalName(LocIn,LocOut);");
      file.Writeln(4,"//------------------ Generation of process body for "+getId()+" -------------------");
      for( ListIterator l = processDef.getCalls().listIterator(); l.hasNext() ; ){
	  ((Call)l.next()).generateJavaCode(file);
      }
      file.Writeln(4,"//----------------------- End of ----process body for "+getId()+" ---------------------");
      // JE: Here we unstack the tmCurrentCompositeProcess
      file.Writeln(4,"tmCurrentCompositeProcess = (TProcessComposite)tmProcessStack.pop();");
    } //end if
  } //end proc

  public void generateLocalName(ReadWrite file) {
  } //end proc

   
} //end class
