/**
 * $Id: ApplyQuery.java,v 1.12 2001/12/20 12:13:34 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcess;

public class ApplyQuery
  extends ApplyImpl {

  public ApplyQuery (String n, String t, Process p, String q){
   super(n, t, p, q, 1/*NbIn */, 1/*NbOut */);
  }

  /** Propagate the nullify of its sole output to all the input. */
  public void retroNull( PortList out, PortList in) {
      nullified = out.getPort(0).getChannel().nullifiedP();
      in.getPort(0).getChannel().setVisited();
      if ( nullified ) in.getPort(0).getChannel().nullify();
      visited = true;
  }
  
  public void generateXML (){
    System.out.print("      <apply-query id=\""+id+"\" type=\""+getType()+"\" ref=\""+getRef()+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    if ( parameters.isEmpty() ){
      System.out.println("\"/>");
    } else {
      System.out.println("\">");
      parameters.generateXMLCalls();
      System.out.println("      </apply-query>");
    }
  }

  public  final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory)
  //throws TransformerException, TransformerConfigurationException, SAXException, IOException 
    {
	String[] vPortIn = inPorts().toStringList();
	String[] vPortOut = outPorts().toStringList();
	// get the name corresponding to the generated stylesheet and pas it in parameter
	String ref = getAttributes().getStringParameter("ref");
	Transmorpher tr = getProcess().getTransmorpher() ;
	String f = tr.getTransformation(ref).getAttributes().getStringParameter("file");
	// static because generate by Transmorpher from the query name
	// JE: This is an error: I am pretty sure that this should be in Parameter (and not Attribute of 
	// the apply-external. It should be the same for TTransformer: in Parameters.
	getAttributes().setParameter("file",f);
	TProcess vCurrent = (TProcess)TProcessFactory.newApplyQuery(vPortIn,vPortOut,
								    getAttributes().getStringParameter("type"),
								    getParameters(),
								    getAttributes());
	currentProcess.addProcess(vCurrent);
    } //end proc

  public void setUp(){
    if( getProcess().getTransmorpher().getTransformation(getRef()) == null ){
      System.err.println("warning : unknown query : "+getRef());
    } //end if
  } //end proc
   
    public void generateJavaCode(ReadWrite file){
	if(file == null) {
	    System.out.println(getId()+" [generateCode]: the file is null!!");
	} else {
	    file.Writeln("");
	    file.Writeln(4,"//------------------Generation of ApplyQuery: "+getId()+" -------------------");
	    generatePorts(file);
	    // Fetch the file to be processed by XSLT (this is very xsltish)
	    String ref = getAttributes().getStringParameter("ref");
	    Transmorpher tr = getProcess().getTransmorpher() ;
	    String f = tr.getTransformation(ref).getAttributes().getStringParameter("file");
	    // Added just for being generated afterwards
	    getAttributes().setParameter("file",f);
	    generateParameters(file);
	    file.Writeln(4,"tmCurrentProcess = tmProcessFactory.newApplyQuery(portIn,portOut,\""+getType()+"\",tmParameters,tmAttributes);");
	    file.Writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
	    file.Writeln("");
	    file.Writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
	} //end if
    } //end proc
    
} //end class
