/**
 * $Id: Process.java,v 1.11 2001/12/20 12:13:34 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph processes definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.util.Stack;
import java.util.Date;

import fr.fluxmedia.transmorpher.Graph.Transmorpher;
import fr.fluxmedia.transmorpher.Graph.Transformation;
import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;
import fr.fluxmedia.transmorpher.TProcessFactory.TApplyProcess;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcess;

public class Process extends TransformationImpl {

  LinearIndexedStruct calls = null;
  Hashtable channels = null;

  public Process(String n, Transmorpher t, int nin, int nout){
    super(n,"Process",t,nin,nout);
    calls = new LinearIndexedStruct();
    channels = new Hashtable(15,5);
  }

  public void addCall( Call c ){
    calls.add(c.getId(), c);
  }

  /**
   * Delete all the calls to a particular transformation
   * name is the name of a transformation (local or imported) to be suppressed.
   */
  public void deleteCallsTo( String name ){
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	Call c = (Call)l.next();
	if ( (c instanceof ApplyImpl) &&
             ((ApplyImpl)c).getRef().equals(name) )
	    removeCall( c );
    }
  }

  /**
   * Delete a call (and the channels connected to this call)
   */
  public void removeCall( Call c ){
    PortList ports = c.inPorts();    
    for ( int i=0; i != ports.length() ; i++) {
      Channel chan = ports.getPort(i).getChannel();
      removeChannel(chan);
    } //end for
    ports = c.outPorts();
    for ( int i=0; i != ports.length() ; i++) {
      Channel chan = ports.getPort(i).getChannel();
      removeChannel(chan);
    } //end for
    calls.remove( c.getId() );
      // the call should not be reachable anymore
  }

  /**
   * Delete a channel
   */
  public void removeChannel( Channel c ){
      if ( c.in() != null ) {
	  if ( c.in().TransformationPortP() ) {
	      c.in().getTransformation().inPorts().removePort(c.in());
	  } else {
	      c.in().getCall().outPorts().removePort(c.in());
	  }
      }
      if ( c.out() != null ) {
	  if ( c.out().TransformationPortP() ) {
	      c.out().getTransformation().outPorts().removePort(c.out());
	  } else {
	      c.out().getCall().inPorts().removePort(c.out());
	  }
      }
      // remove the channel
      channels.remove( c.getName() );
      // the channel should not be reachable anymore
  }

  public LinearIndexedStruct getCalls(  ){
    return calls;
  }
  public Call getCall( String c ){
    return (Call)calls.get(c);
  }

  public void addChannel( Channel c ){
    channels.put(c.getName(), c);
  }

  public Enumeration getChannels(){
    return channels.elements();
  }

  public Channel getChannel( String c ){
    return (Channel)channels.get(c);
  }
  
  public void retroNull(PortList out, PortList in) {
      retroNull(out, in, true);
  }

  // Problem with the STACK
    // The boolean is only used for selecting the method
  public boolean retroNull(PortList out, PortList in, boolean b) {
    Stack stack = new Stack();
    for ( int i=0; i != out.length(); i++ ) {
	outPorts.getPort(i).getChannel().setVisited();
	if( out.getPort(i).getChannel().nullifiedP() ) outPorts.getPort(i).getChannel().nullify();
	// stack.push(outPorts.getPort(i).getChannel().);
    }
    retroNull();
    // report on in
    boolean nullified = true;
    for ( int i=0; i != in.length(); i++ ) {
	in.getPort(i).getChannel().setVisited();
	if( inPorts.getPort(i).getChannel().nullifiedP() ) {
	    in.getPort(i).getChannel().nullify();
	} else {
	    nullified = false;
	}
    }
    return nullified;
  }
  
  public void retroNull() {
    Stack stack = new Stack();
    for ( Enumeration e = getChannels(); e.hasMoreElements() ; ){
	Channel c = (Channel)e.nextElement();
	if (c instanceof Null) {
	    c.setVisited();
	    if ( c.in().CallPortP() ) stack.push(c.in().getCall());
	}
    }
    // enumeration of the Stack
    /*
    for (;;) {
	Call c = (Call)stack.pop();
	if () // all in are visited
	  c.retroNull();
    }
    */
    clearVisited();
  }
  
  public void clearNull() {
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).clearNull();
    }
    for( Enumeration e = getChannels(); e.hasMoreElements() ; ){
      ((Channel)e.nextElement()).clearNull();
    }
  }
  
  public void clearVisited() {
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).clearVisited();
    }
    for( Enumeration e = getChannels(); e.hasMoreElements() ; ){
      ((Channel)e.nextElement()).clearVisited();
    }
  }

  private String XMLName(){
    return "process";
  }

  /** Prints the XML expression of the process */
  public void generateXML(){
    System.out.print("  <"+XMLName()+" name=\""+name+"\" type=\""+type+"\" in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    System.out.println("\">");
    parameters.generateXMLDeclarations();
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).generateXML();
    }
    System.out.println("  </"+XMLName()+">");
    System.out.println();
  }

    public void setUp(){
	for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).setUp();
	}
    }

    public void generateJavaCode(ReadWrite iFile,LinearIndexedStruct externs){};

} //end class
