/**
 * $Id: MapAtt.java,v 1.2 2001/09/27 07:24:52 laurent Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph rule node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph.Rules ;

import fr.fluxmedia.transmorpher.Files.ReadWrite;

public class MapAtt implements Rule {

  public String match = null;
  public String target = null;
  public String context = null;

  public MapAtt(String m, String t){
    super();
    match = m;
    target = t;
  }

  public MapAtt(String m, String t, String c){
    this(m, t);
    context = c;
  }

  /** Prints the XML expression of the rule */
  public void generateXML(){
    System.out.print("    <mapatt match=\""+match+"\"");
    if ( context != null ) System.out.print(" context=\""+context+"\"");
    System.out.println("target=\""+target+"/>");
  }

  public void generateXSLTCode(ReadWrite file){
    file.Writeln(2,"<!-- Mapping attributes "+match+" to "+target+" -->");
    file.Write(2,"<xsl:template match=\"");
    if( context != null ) file.Write(context+"/");
    file.Writeln("@"+ match+"\">");
    file.Writeln(4,"<xsl:attribute name=\""+target+"\">");
    file.Write("<xsl:value-of select=\"@"+match+"\"/>");
    file.Writeln(4,"</xsl:attribute>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");
  }

  /** Prints the XSLT code inside a ModTag template */
  public void generateInsideXSLTCode(ReadWrite file){
    file.Writeln(6,"<!-- Mapping attributes "+match+" to "+target+" -->");
    file.Writeln(6,"<xsl:attribute name=\""+target+"\">");
    file.Write("<xsl:value-of select=\"@"+match+"\"/>");
    file.Writeln(6,"</xsl:attribute>");
    file.Writeln("");
  }

  /** Prints the Java code for the execution of the rule */
  public void generateJavaCode(ReadWrite file){
    System.out.println("[MapAtt]generateJavaCode : not yet implemented");
  }

}
