/**
 * $Id: Servlet.java,v 1.11 2001/12/20 16:00:09 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph servlet definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;

public class Servlet extends MainProcess {

  public Servlet(String name, Transmorpher t){
    super(name, t, 0, 0);
  }
  
  public void retroNull() {
  }

  /** Prints the XML expression of the servlet */
  public void generateXML(){
    System.out.println("  <servlet name=\""+name+"\">");
    parameters.generateXMLDeclarations();
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).generateXML();
    }
    System.out.println("  </servlet>");
    System.out.println();
  }

  public TProcessComposite createMainProcess(TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newServlet(vPortIn,vPortOut,getParameters());
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	((Call)l.next()).createProcess(vCurrent,iProcessFactory);
    }
    return vCurrent ;
  }

    public void generateImport(ReadWrite iFile,LinearIndexedStruct externs){
	iFile.Writeln("//import Servlet classes");
	iFile.Writeln("import java.io.*;");
	iFile.Writeln("import javax.servlet.*;");
	iFile.Writeln("import javax.servlet.http.*;");
	iFile.Writeln();
    }
 
    public void generateClassHeader(ReadWrite iFile) {
	iFile.Writeln("public class "+getName()+" extends HttpServlet {");
    }

    public void generateMainHeader(ReadWrite iFile) {
	iFile.Writeln(2,"public void init(ServletConfig config) throws ServletException{");
	iFile.Writeln(4,"try{");
	iFile.Writeln(6,"vSR= new SystemResources();");
	iFile.Writeln(6,"tmProcessStack = new Stack();");
	iFile.Writeln(6,"tmProcessFactory = new TProcessFactory(newDefexterns(),debug_mode);");
	iFile.Writeln(4,"} catch (Exception TransformException) {");
	iFile.Writeln(4,"}");
	iFile.Writeln(2,"}");
	iFile.Writeln();
	iFile.Writeln(2,"protected void doGet(HttpServletRequest req, HttpServletResponse res)");
	iFile.Writeln(4,"          throws ServletException, IOException {");
	iFile.Writeln(4,"   doPost(req,res);");
	iFile.Writeln(2,"}");
	iFile.Writeln();
	iFile.Writeln(2,"protected void doPost(HttpServletRequest req, HttpServletResponse res)");
	iFile.Writeln(4,"          throws ServletException, IOException {");
	iFile.Writeln();
    }
 
    public void generateReadParameters(ReadWrite iFile) {
	// JE: Here are only considered the parameters that have been declared
	iFile.Writeln(6,"tmParameters = new Parameters();");
	for( Enumeration e = getParameters().getNames() ; e.hasMoreElements(); ){
	    Object key = e.nextElement();
	    iFile.Writeln(6,"if ( req.getParameter(\""+key+"\") != null ){");
	    iFile.Writeln(8,"tmParameters.setParameter(\""+key+"\",req.getParameter(\""+key+"\"));");
	    iFile.Writeln(6,"};");
	} //end for
	iFile.Writeln();
	iFile.Writeln(6,"try{");
	iFile.Writeln(8,"res.setContentType(\"text/plain\");");
	iFile.Writeln(8,"PrintWriter out = res.getWriter();");
    }
 
    public void generateBody(ReadWrite iFile,LinearIndexedStruct externs) {
	Parameters param = getParameters();
	iFile.Writeln(4,"Parameters p = new Parameters();");
	for( Enumeration e = param.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    iFile.Writeln(4,"p.setParameter(\""+key+"\", \""+(String)param.getParameter(key)+"\");");
	}
	iFile.Writeln(4,"tmCurrentCompositeProcess = (TServlet)tmProcessFactory.newMain(new String[0], new String[0], p);");
	for( ListIterator l = getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(iFile);
	}
	// JE: warning, reuse of parameters
	iFile.Writeln(4,"tmCurrentCompositeProcess.bindParameters(p);");
	iFile.Writeln(4,"tmCurrentCompositeProcess.execProcess();");
    }
 
    public void generateEnd(ReadWrite iFile) {
	iFile.Writeln(4,"if (debug_mode>0) {");
	iFile.Writeln(6,"end = System.currentTimeMillis();");
	iFile.Writeln(6,"System.err.println(\">> End of execution (\"+(end-begin)+\")\");}");
	iFile.Writeln(4,"} catch (Exception TransformException) {");
	iFile.Writeln(4,"}");
	iFile.Writeln(2,"} //end main");
	iFile.Writeln("} //end class");
    }

} //end class
