/**
 * $Id: Transformer.java,v 1.9 2001/12/20 16:00:09 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph (JAXP) transformer definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;

import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;

public class Transformer extends MainProcess {

  public Transformer(String name, Transmorpher t){
    super(name,t,1,1);
  }

  public Transformer(Transmorpher t){
    this("transformer", t);
  }

  private String XMLName(){ return "transformer";}

  public TProcessComposite createMainProcess(TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newTransformer(vPortIn,vPortOut,getParameters());
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	((Call)l.next()).createProcess(vCurrent,iProcessFactory);
    }
    return vCurrent ;
  }
    public void generateImport(ReadWrite iFile,LinearIndexedStruct externs){
	iFile.Writeln("//import JAXP classes");
	iFile.Writeln("import javax.*;");
	iFile.Writeln();
    }
 
    public void generateClassHeader(ReadWrite iFile) {
	iFile.Writeln("public class "+getName()+" implements javax.Transformer {");
    }


    public void generateMainHeader(ReadWrite iFile) {
	// JE: Here, this should not be main but:
	//iFile.Writeln(2,"public static void transform(StreamSource in, StreamResult out){");
	iFile.Writeln(2,"public static void main(String[] args){");
	iFile.Writeln(4,"tmProcessFactory = new TProcessFactory(newDefexterns(),debug_mode);");
	iFile.Writeln(4,"tmProcessStack = new Stack();");
	iFile.Writeln(4,"vSR = new SystemResources();");
	iFile.Writeln();
    }
 
    public void generateReadParameters(ReadWrite iFile) {
	// JE: Here are only considered the parameters that have been declared
	for( Enumeration e = getParameters().getNames() ; e.hasMoreElements(); ){
	    Object key = e.nextElement();
	    // JE: Here I must find the code corresponding to the transformer
	    iFile.Writeln(4,"if ( req.getParameter(\""+key+"\") != null ){");
	    iFile.Writeln(6,"tmParameters.setParameter(\""+key+"\",req.getParameter(\""+key+"\"));");
	    iFile.Writeln(4,"};");
	} //end for
	iFile.Writeln();
	iFile.Writeln(4,"res.setContentType(\"text/plain\");");
	iFile.Writeln(4,"PrintWriter out = res.getWriter();");
	iFile.Writeln();
    }
 
    public final void generateBody(ReadWrite iFile,LinearIndexedStruct externs){
	Parameters param = getParameters();
	iFile.Writeln(4,"Parameters p = new Parameters();");
	for( Enumeration e = param.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    iFile.Writeln(4,"p.setParameter(\""+key+"\", \""+(String)param.getParameter(key)+"\");");
	}
	iFile.Writeln(4,"tmCurrentCompositeProcess = (TMain)tmProcessFactory.newMain(new String[0], new String[0], p);");
	for( ListIterator l = getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(iFile);
	}
	// JE: warning, reuse of parameters
	iFile.Writeln(4,"tmCurrentCompositeProcess.bindParameters(p);");
	iFile.Writeln(4,"tmCurrentCompositeProcess.execProcess();");
    } //end proc

    public void generateEnd(ReadWrite iFile) {
	iFile.Writeln(4,"if (debug_mode>0) {");
	iFile.Writeln(6,"end = System.currentTimeMillis();");
	iFile.Writeln(6,"System.err.println(\">> End of execution (\"+(end-begin)+\")\");}");
	iFile.Writeln(2,"} //end main");
	iFile.Writeln("} //end class");
    }

} //end class
