/**
 * $Id: Transmorpher.java,v 1.21 2001/12/20 16:00:09 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.Graph;

// import JSDK classes

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.ListIterator;

import java.io.IOException;
import java.io.File;

import org.xml.sax.SAXException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

// Transmorpher imports

import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.Version;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Files.Constants ;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory ;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite ;

public class Transmorpher {

 /** The name of the Transmorpher sheet */
  protected String name = null ;
    
  /** The filename of the Transmorpher sheet */
  protected File href = null ;
    
  /** The main process of the Transmorpher sheet */
  protected MainProcess main = null ;
    
  /** the list of Processes */
  protected LinearIndexedStruct transformations = null ;

  /** the version to be used */
  String version = null ;

  /* the execution structure */
  TProcessComposite iExecutionStructure = null;
  
  /* the process factory used for generating the execution structure */
  TProcessFactory iProcessFactory = null;

  /** the list of imports
      this could be a list of urls OR of Transmorpher
      if the import has been parsed. In such a case the URL can be found
      in its href attribute */
  protected LinkedList imports = null ;

  /** the list of includes */
  protected LinkedList includes = null ;

  /** the list of declared names of components */
  protected LinearIndexedStruct defexterns = null ;

  int debug = 0;
  boolean opt = false;

  public Transmorpher(int d){
  // JE: name might be compulsory...
    debug = d;
    transformations = new LinearIndexedStruct();
    defexterns = new LinearIndexedStruct();
    imports = new LinkedList();
    includes = new LinkedList();
  }

  public Transmorpher(String n, String v, int d, boolean o){
    this(d);
    opt = o;
    name = n;
    href = new File( name+".xml" );
    version = v;
    if (!version.equals(Version.VERSION) ) 
	System.err.println("Warning : bad version "+version+" instead of "+Version.VERSION);
  }

    // This is a Transmorpher inheriting some externs...
  public Transmorpher(String n, String v, int d, boolean o, Transmorpher t){
    this(d);
    opt = o;
    name = n;
    href = new File( name+".xml" );
    version = v;
    if (!version.equals(Version.VERSION) ) 
	System.err.println("Warning : bad version "+version+" instead of "+Version.VERSION);
    defexterns = t.getDefexterns();
  }

  public final String getName(){
    return name;
  }
    
  public final void setName(String n){
    name = n;
  }
    
  public final File getFile(){
    return href;
  }
    
  public final void setFile(File u){
      href = u;
  }
    
  public final String getVersion(){
    return version;
  }
    
  public final void setVersion(String v){
    version = v;
  }
    
  public final boolean optimizedP(){
    return opt;
  }
    
  public final void setOptimized(boolean b){
    opt = b;
  }
    
  public final MainProcess getMain(){
    return main;
  }
    
  public final Transformation getTransformation(String name){
    return (Transformation)transformations.get(name);
  }

  public final void addTransformation(String name, Transformation t){
    // JE: If it already exists one == error (this wil cause filename problems)
    transformations.add(name, (Object)t);
  }
    
  public final void removeTransformation( Transformation trans ) {
      transformations.remove( trans.getName() );
      // Check that all the processes defined here do not use the transformation.
      // If so, suppress it from the transformation p.deleteCall( c )
      if ( !(trans instanceof MainProcess) ) {
	 for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	   Transformation t = ((Transformation)l.next());
	   if (t instanceof Process) ((Process)t).deleteCallsTo( trans.getName() );
	 } // end for
       } // end if
      // Take care of the main
      if ( main == (MainProcess)trans ) main = null;
      // the transformation should not be reachable anymore
  }
    
  public final String getExtern(String name){
    return (String)defexterns.get(name);
  }

  public final void addExtern(String name, String cl, String impl, boolean def){
      if ( getExtern(name) != null ){
	  System.err.println("Error: "+name+" external already declared");
      } else {
	  defexterns.add(name, (Object)cl);
      }
  }
    
  public final void removeExtern( String name ) {
      defexterns.remove( name );
      // Check that all the processes defined here do not use the external call.
      // If so, suppress it from the transformation p.deleteCall( c )
      for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	   Transformation t = ((Transformation)l.next());
	   if (t instanceof Process) ((Process)t).deleteCallsTo( name );
      } // end for
  }
    
  public final LinearIndexedStruct getDefexterns() { return defexterns; }
    
  public final void addImport(String i){
    imports.add((Object)i);
  }

  public final LinkedList getImports(){
    return imports;
  }
    
  public final void addInclude(String i){
    includes.add((Object)i);
  }

  public final LinkedList getIncludes(){
    return includes;
  }
    
  public final void setMain(MainProcess p){
    main = p;
  }

  public final void retroNull(){
    main.retroNull();
  }

  public final void generateXML(){
    System.out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    System.out.println("<!DOCTYPE transmorpher SYSTEM \"http://transmorpher.inrialpes.fr/dtd/transmorpher.dtd\">");
    System.out.println("<transmorpher name=\""+name+"\"");
    System.out.println("              version=\""+version+"\"");
    System.out.println("              optimized=\""+opt+"\"");
    System.out.println("              xmlns=\"http://transmorpher.fluxmedia.fr/1.0\"");
    System.out.println("              xmlns:regexp=\"xalan://fr.fluxmedia.transmorpher.regexp.RegularExpression\">");
    System.out.println();
    for (ListIterator l = defexterns.listIterator(); l.hasNext();) {
	   String name = ((String)l.next());
      System.out.println("<defextern name=\""+name+"\" class=\""+(String)defexterns.get(name)+"\"/>");
    } //end for
    for (ListIterator l = imports.listIterator(0); l.hasNext();) {
      System.out.println("<import href=\""+(String)l.next()+"\"/>");
    } //end for
    for (ListIterator l = includes.listIterator(0); l.hasNext();) {
      System.out.println("<include href=\""+(String)l.next()+"\"/>");
    }
    for (ListIterator l = transformations.listIterator(); l.hasNext();) {
      Transformation t = ((Transformation)l.next());
      if ( t  != main ){
        t.generateXML();
      } //end if
    } //end for
   if ( main != null ) main.generateXML();
    System.out.println("</transmorpher>");
  } //end generate XML


  /** Generates Java code for the transformation **/
  public final void generateExec(String reloc)
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException {
    for (ListIterator l = transformations.listIterator(); l.hasNext();) {
      Transformation t = ((Transformation)l.next());
      if (t instanceof Ruleset) ((Ruleset)t).generateXSLTStylesheet(reloc);
      else if (t instanceof Query) ((Query)t).generateXSLTStylesheet(reloc);
    } //end for
     if ( getMain() != null ){
	  iProcessFactory = new TProcessFactory(defexterns,debug);
	  iExecutionStructure = getMain().createMainProcess(iProcessFactory);
	  // JE: This should include the generation of the import/include!
	  // So far, this is not absolutely easy
      } else {
	  System.err.println("Cannot generate execution structure for mainless Transformation");
      }
  } //end generateExec

    public final void generateJavaCode(String reloc) {
	String iCompilDirectory = null;
	ReadWrite iFile = null;

	for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	    Transformation t = ((Transformation)l.next());
	    if (t instanceof Ruleset) ((Ruleset)t).generateXSLTStylesheet(reloc);
	    else if (t instanceof Query) ((Query)t).generateXSLTStylesheet(reloc);}
	try {
	    iCompilDirectory = getFile().getAbsoluteFile().getParent();
	    if( reloc != null) iCompilDirectory = reloc;
	    else iCompilDirectory = getFile().getAbsoluteFile().getParent();
	    File dir = new File(iCompilDirectory);
	    dir.mkdir(); // directly tries to create the directory that might exist
	    iCompilDirectory+=File.separator;
	    iFile = new ReadWrite(iCompilDirectory+getMain().getName()+".java", Constants.WRITE);
	    getMain().generateJavaCode(iFile,defexterns);
	} catch (Exception e) {
	    System.out.println("Error:" + e );
	} //end try
	iFile.Close();
    } //end generate JavaCode

  public final void exec(Parameters p) throws Exception {
     if ( iExecutionStructure != null ){
	  iExecutionStructure.bindParameters(p);
	  iExecutionStructure.execProcess();
     }
  } //end run

}//end class
