/**
 * $Id: TDispatcher.java,v 1.14 2001/12/12 13:11:13 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * this class define the dispatcher process.
  * dispatch one data flow to different process.
  *
  * @ author laurent tardif
  * @date 10 02 2001
  * @since jdk 1.2 SAX 2.0
  */

package fr.fluxmedia.transmorpher.TProcessFactory ;

// Imported transmorpher classes
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.Parameters;

// Imported SAX classes

import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.Locator ;
import org.xml.sax.SAXException;

import java.util.Hashtable;

public final class TDispatcher
  extends TProcessBasic
  implements ContentHandler {
    
    /** a list of content handler to dispatch information */
  ContentHandler [] iListOfContent = null ;
    
  int i = 0;
    
     
  /** the constructor */
   /*
   public TDispatcher(String[] pIn, String[] pOut){
  
    super(pIn,pOut);
    iListOfContent = new ContentHandler[iNbOut];
  }
  */
  
  /** the constructor */
  public TDispatcher(String[] pIn, String[] pOut,Parameters pParam){
    super(pIn,pOut);
    type= (String)pParam.getParameter("type");
    iListOfContent = new ContentHandler[iNbOut];
  }
        
  /** set a content handler */
  public final void setContentHandler(int i, ContentHandler pContentHandler) {
  
    iListOfContent[i] = pContentHandler ;
  }
    
    // a set of method to implement content handler interface
    
  /** the start element method of content handler interface */
  public final void startElement(String ns,String localName,String name, Attributes atts) throws SAXException {
  //System.out.println("dispatcher "+this);
  //System.out.println("[TDispatcher["+getName()+"] startElement =>" +ns +","+localName+","+name+"numero :"+i++);
  /*if (atts != null) {
    int len = atts.getLength();
    for (int i = 0; i < len; i++) {
    System.out.print("AttName="+atts.getQName(i));
    System.out.println("Value="+atts.getValue(i));
    }
    }*/
  
    for (int i=0; i<iNbOut; i++) {
      //  System.out.println("content handler "+iListOfContent[i]);
      iListOfContent[i].startElement( ns, localName, name,  atts);
    }
  }
    
  /** the endElement method of content handler interface */
  public final void endElement(String ns,String localName,String name) throws SAXException {
    //System.out.println("[TDispatcher] endElement=>" +ns +","+localName+","+name);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].endElement( ns, localName, name);
    }
  }

  /** the setDocumentLocator method of content handler interface */
  public final void setDocumentLocator(Locator locator) {
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].setDocumentLocator( locator);
    }
  }

  /** the startDocument method of content handler interface */
  public final void startDocument() throws SAXException {
    //System.out.println("[TDispatcher] startDocument=>");
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].startDocument();
    }
  }

  /** the endDocument method of content handler interface */
  public final void endDocument() throws SAXException {
    
      for (int i=0; i<iNbOut; i++) {
        iListOfContent[i].endDocument();
      }
  }

  /** the skippedEntity method of content handler interface */
  public final void skippedEntity(java.lang.String name) throws SAXException {
    //System.out.println("[TDispatcher] skippedEntity=>" +name);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].skippedEntity( name);
    }
  }

  /** the processingInstruction method of content handler interface */
  public final void processingInstruction(java.lang.String target, java.lang.String data) throws SAXException {
    //System.out.println("[TDispatcher] processingInstruction=>" +target+","+data);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].processingInstruction(target, data);
    }
  }

  /** the ignorableWhitespace method of content handler interface */
  public final void ignorableWhitespace(char[] ch, int start, int length) throws SAXException{
   /*String titi="";
    for (int i=start; i<start+length; i++){
      titi+=ch[i];
    }
    System.out.println("ignorableWhitespace" + titi);
    */
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].ignorableWhitespace( ch,  start,  length);
    }
  }

  /** the characters method of content handler interface */
  public final void characters(char[] ch, int start, int length) throws SAXException{
    /*String titi="";
    for (int i=start; i<start+length; i++){
      titi+=ch[i];
    }
    System.out.println("characters" + titi);
    */
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].characters( ch,  start,  length);
    }
  }

  /** the startPrefixMapping method of content handler interface */
  public final void startPrefixMapping(java.lang.String prefix, java.lang.String uri) throws SAXException{
    //System.out.println("[TDispatcher] startPrefixMapping=>" +prefix+","+uri);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].startPrefixMapping( prefix,uri);
    }
  }

  /** the endPrefixMapping method of content handler interface */
  public final void endPrefixMapping(java.lang.String prefix) throws SAXException{
    //System.out.println("[TDispatcher] endPrefixMapping=>" +prefix);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].endPrefixMapping( prefix);
    }
  }

  /**
     The copy of this element. The method was call by the copy method
     of the TProcess_impl class
  */
//
//  public final TProcess internalCopy() {
//    TDispatcher vAns = new TDispatcher(nameIn,nameOut,type);
//    vAns.setFatherName(this.getFatherName());
//    return vAns ;
//  }
//

  public void generatePort(){
    //System.out.println("[TDISPATCHER] generatePort : " + iNbOut);
    if(nameFather != null) {
      for (int i=0; i<iNbOut; i++) {
        iListOut [i] = new XML_Port(nameFather+"-"+nameOut[i],this,this,i,XML_Port.OUT);
       } //end for
      iListIn[0]= new XML_Port(nameFather+"-"+nameIn[0],this,(ContentHandler)this,0,XML_Port.IN);
    } else {
      for (int i=0; i<iNbOut; i++) {
        iListOut [i] = new XML_Port(nameOut[i],this,this,i,XML_Port.OUT);
      } //end for
      iListIn[0]= new XML_Port(nameIn[0],this,(ContentHandler)this,0,XML_Port.IN);
    } //end if
  } //end proc
} //end class
