/**
 * $Id: Parameters.java,v 1.5 2001/12/20 12:09:16 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/** define the Process parameter structures
*/

package fr.fluxmedia.transmorpher.Utils ;

// import java classes
import java.util.Hashtable;
import java.util.Enumeration;

public class Parameters {
 
  /** The list of unlinked out  XML_Port */
  Hashtable parameters = null;
    
  public Parameters() {
    parameters = new Hashtable();
  }
  
  public Parameters(Parameters toCopy) {
    parameters = new Hashtable();
    setParameters(toCopy);
  }
 
  public boolean isEmpty(){
    return parameters.isEmpty();
  }

  public void setParameter(String name, Object value){
    parameters.put((Object)name,value);
  }

  public void setParameters(Parameters toCopy){
    for( Enumeration e = toCopy.getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      String val = (String)toCopy.getParameter(key);
      if ( val != null ) setParameter( key, val );
    } //end for
  }
 
  public void unsetParameter(String name){
    parameters.remove((Object)name);
  }

  public Object getParameter(String name){
    return parameters.get((Object)name);
  }

  public Enumeration getNames(){
    return parameters.keys();
  }

  public String evalString( String s ){
      String result = "";
      int first = 0;
      int index = 0;
      String name = null;
      String value = null;

      for( ; index != -1 ; ){
	  index = s.indexOf('$',first);
	  if ( index == -1 ) result = result+s.substring(first);
	  else if ( ( index != 0 && s.charAt(index-1) == '\\') ){
	      result = result+s.substring(first,index+1);
	      first = index+1;
	  } else {
	      result = result+s.substring(first,index);
	      index++;
	      for( ; (index < s.length()) && (s.charAt(index) == ' ' || s.charAt(index) == '\t') ; index++ );
	      if (s.charAt(index) == '{'){
		  first = index+1;
		  index = s.indexOf('}',first);
		  name = s.substring(first,index);
		  if ( index == s.length() ) first = -1;
		  else first = index+1;
	      } else {
		  name = s.substring(index);
		  index = -1;
	      } // end if
	      value = (String)getParameter(name);
	      if ( value != null ) result = result+value;
	      else System.err.println("Variable not bound: "+name);
	  } // end if
      } // end for
      return result;
  }

      /** this does not create a new parameter structure but modifies the arguments */

    // Eval the parameters in a parameter list
  public Parameters bindParameters( Parameters p ){
    for( Enumeration e = p.getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      String val = (String)p.getParameter(key);
      //System.err.println("Set "+key+" to "+evalString(val));
      if ( val != null ) p.setParameter( key, evalString(val) );
    } //end for
    return p;
  }

    // Replace the parameters in a parameter list, if they are not defined, eval their default
  public Parameters bindCallerParameters( Parameters p ){
    for( Enumeration e = p.getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      String val = (String)getParameter(key); // resolve the variable
      if ( val == null ) val = evalString((String)p.getParameter(key)); // get its default value
      //System.err.println("Set "+key+" to "+val);
      if ( val != null ) p.setParameter( key, val );
    } //end for
    return p;
  }

  public void generateXMLDeclarations (){
    for( Enumeration e = parameters.keys() ; e.hasMoreElements(); ){
      Object key = e.nextElement();
      String val = (String)parameters.get(key);
      if ( val == null ) val = "";
      System.out.print("    <param name=\""+(String)key+"\">"+val+"</param>");
    } //end for
  }
  public void generateXMLCalls (){
    for( Enumeration e = parameters.keys() ; e.hasMoreElements(); ){
      Object key = e.nextElement();
      System.out.println("        <with-param name=\""+(String)key+"\">"+(String)parameters.get(key)+"</with-param>");
    } //end for
  } //end proc
 
  public void printParameters (){
    System.out.println("[");
    for( Enumeration e = parameters.keys() ; e.hasMoreElements(); ){
      Object key = e.nextElement();
      System.out.println((String)key+"="+(String)parameters.get(key)+";");
    } //end for
    System.out.println("]");
  } //end proc
 
} //end class

